<?php
defined('BASEPATH') or exit('No direct script access allowed');

/**
 * 指定条件触发奖励
 */
class RewardTrigger extends MY_Controller
{
    /**
     * 逻辑运算符 ID
     */
    const LOGICAL_AND = 'and';      // 与
    const LOGICAL_OR = "or";        // 或

    const ALL_LOGICAL = [
        self::LOGICAL_AND => "&&",
        self::LOGICAL_OR => "| |",
    ];

    /**
     * 关系运算符 ID
     */
    const RELATIONAL_GT = "gt";  // 大于
    const RELATIONAL_LT = "lt";  // 小于
    const RELATIONAL_EQ = "eq";  // 等于
    const RELATIONAL_NE = "ne";  // 不等于
    const RELATIONAL_GE = "ge";  // 大于等于
    const RELATIONAL_LE = "le";  // 小于等于

    const ALL_RELATIONAL = [
        self::RELATIONAL_GT => ">",
        self::RELATIONAL_LT => "<",
        self::RELATIONAL_EQ => "==",
        self::RELATIONAL_NE => "! =",
        self::RELATIONAL_GE => ">=",
        self::RELATIONAL_LE => "<=",
    ];

    /**
     * 触发场景ID
     */
    const SCENE_LOGIN = 1;  // 用户登录
    const SCENE_PAY = 2;    // 用户付费
    const SCENE_UPGRADE = 3;    // 世界升级

    const ALL_SCENE = [
        self::SCENE_PAY => "用户付费",
        self::SCENE_LOGIN => "用户登录",
        self::SCENE_UPGRADE => "世界升级",
    ];

    /**
     * 触发数据点 ID
     */
    const CONDITION_USER_LV = 1001;             // 玩家等级
    const CONDITION_LV_STAY_DAYS = 1002;        // 当前等级停留天数
    const CONDITION_PAY_COUNT = 1003;           // 付费总额
    const CONDITION_LOSE_DAYS = 1004;           // 流失天数
    const CONDITION_PAY_SILENCE_DAYS = 1005;    // 付费沉默天数
    const CONDITION_PAY_TIMES = 1006;           // 付费次数
    const CONDITION_PAY_AMOUNT = 1007;          // 当次付费金额
    const CONDITION_UID_TAIL = 1009;            // UID尾号

    const ALL_CONDITION = [
        self::CONDITION_USER_LV => "玩家等级",
        self::CONDITION_LV_STAY_DAYS => "当前等级停留天数",
        self::CONDITION_PAY_COUNT => "付费总额",
        self::CONDITION_LOSE_DAYS => "流失天数",
        self::CONDITION_PAY_SILENCE_DAYS => "付费沉默天数",
        self::CONDITION_PAY_TIMES => "付费次数",
        self::CONDITION_PAY_AMOUNT => "当次付费金额",
        self::CONDITION_UID_TAIL => "UID尾号",
    ];

    /**
     * @var array Post 数据
     */
    private $postData;

    public function __construct()
    {
        parent::__construct();

        $this->postData = $this->input->post();
    }

    /**
     * 配置列表
     */
    public function index()
    {
        $tb = otable::rewardTriggerConfig();

        $sql = "SELECT * from {$tb} WHERE 1";
        $triggerConf = oo::commonOprDb('games')->getAll($sql, 1);

        $data = [];
        $data['sceneList'] = self::ALL_SCENE;
        $data['conditionList'] = self::ALL_CONDITION;
        $data['logicalList'] = self::ALL_LOGICAL;
        $data['relationalList'] = self::ALL_RELATIONAL;
        $data['switch'] = oo::commonOprModel('config')->getCon('rewardTrigger', "switch");
        $data['list'] = $this->formatTriggerConf($triggerConf);

        $setInterval = oo::commonOprModel('config')->getCon('rewardTrigger', "setInterval");
        $setInterval = json_decode($setInterval, true);
        foreach ($setInterval as &$item) {
            $item['interval'] = round($item['interval'] / 86400, 5);
        }

        $data['setInterval'] = json_encode($setInterval);

        $this->load->view("rewardTrigger/index", $data);
    }

    /**
     * 触发奖励配置变动
     */
    public function updateConfig()
    {
        $formData = $this->postData;

        foreach ($formData['condition'] as &$item) {
            if (in_array($item['cid'], [self::CONDITION_PAY_COUNT, self::CONDITION_PAY_AMOUNT])) {
                $item['num'] = ceil($item['num'] * 100);
            }
        }
        foreach ($formData as $key => $one) {
            if (is_array($one)) {
                $formData[$key] = json_encode($one, JSON_NUMERIC_CHECK);
            }
        }

        $tb = otable::rewardTriggerConfig();
        // 新增配置
        if (empty($formData['id'])) {
            $sql = funs::db_insertSQL($tb, $formData);
        } else {
            // 修改配置
            $sql = funs::db_updateSQL($tb, $formData, "id = " . $formData['id']);
        }

        $ret = oo::commonOprDb('games')->query($sql);

        if (!$ret) {
            return $this->response(1, '操作失败');
        } else {
            return $this->response(0, '操作成功');
        }
    }

    public function updateSwitch()
    {
        $formData = $this->postData;

        $ret = oo::commonOprModel('config')->updateCon("rewardTrigger", "switch", $formData['switch'], '奖励触发功能总开关');

        if (!$ret) {
            return $this->response(1, '操作失败');
        } else {
            return $this->response(0, '操作成功');
        }
    }

    public function updateSetInterval()
    {
        $formData = $this->postData;


        $set = [];
        foreach ($formData['conf'] as $id => &$item) {
            foreach ($item['set'] as $one) {
                if (in_array($one, $set)) {
                    $idc = $id + 1;
                    return $this->response(1, "集合ID:{$idc}, 存在重复触发ID");
                }

                $set[] = $one;
            }
            $item['interval'] = ceil($item['interval'] * 86400);
        }

        $conf = json_encode(array_values($formData['conf']), JSON_NUMERIC_CHECK);
        $ret = oo::commonOprModel('config')->updateCon("rewardTrigger", "setInterval", $conf, '奖励触发功能触发集合间隔');

        if (!$ret) {
            return $this->response(1, '操作失败');
        } else {
            return $this->response(0, '操作成功');
        }
    }

    public function formatTriggerConf(array $triggerConf): array
    {
        foreach ($triggerConf as &$item) {
            $item['condition'] = json_decode($item['condition'], true);
            foreach ($item['condition'] as &$item2) {
                if (in_array($item2['cid'], [self::CONDITION_PAY_COUNT, self::CONDITION_PAY_AMOUNT])) {
                    $item2['num'] = round($item2['num'] / 100, 2);
                }
            }

            $item['conditionHtml'] = $this->buildConditionHtml($item['condition']);
            $item['condition'] = json_encode($item['condition']);
            $item['switchHtml'] = $this->buildSwitchHtml($item['switch']);
            $item['sceneHtml'] = $this->buildSceneHtml($item['scene']);
        }
        return $triggerConf;
    }

    public function buildConditionHtml($condition): string
    {
        $len = count($condition);

        $html = '';
        foreach ($condition as $index => $item) {
            $logicOp = ($len == $index + 1) ? '' : self::ALL_LOGICAL[$item['logicalOp']];
            $html .= self::ALL_CONDITION[$item['cid']] . ' ' . self::ALL_RELATIONAL[$item['relationalOp']] . ' ' . $item['num'] . ' ' . $logicOp . ' ';
        }

        return $html;
    }

    public function buildSwitchHtml($switch): string
    {
        $bgc = "#409EFF";
        $str = "上线";
        if (empty($switch)) {
            $str = "下线";
            $bgc = "#F56C6C";
        }

        return "<span style='margin-top: 2px;height: 30px;font-size: 12px;background:$bgc' class='btn btn-primary'>$str</span>";

    }

    public function buildSceneHtml($scene): string
    {
        $str = self::ALL_SCENE[$scene];
        return "<span style='margin-top: 2px;height: 30px;font-size: 12px;background: #337ecc' class='btn btn-primary'>$str</span>";
    }

}
