<?php
defined('BASEPATH') or exit('No direct script access allowed');

/**
 * 场景触发
 */
class SceneTrigger extends MY_Controller
{
    /**
     * 触发场景 ID
     */
    const SCENE_SPINS_SHORTAGE = 1;
    const SCENE_BUILD_COIN_SHORTAGE = 2;
    const SCENE_SLOT_COIN_SHORTAGE = 3;
    const SCENE_EASTER_COIN_SHORTAGE = 4;

    const ALL_SCENE = [
        self::SCENE_SPINS_SHORTAGE => "体力不足",
        self::SCENE_BUILD_COIN_SHORTAGE => "建造金币不足",
        self::SCENE_SLOT_COIN_SHORTAGE => "金币老虎机金币不足",
        self::SCENE_EASTER_COIN_SHORTAGE => "四选一复活金币不足",
    ];

    /**
     * @var array Post 数据
     */
    private $postData;

    /**
     * @var string 数据库表名
     */
    private $tb;

    public function __construct()
    {
        parent::__construct();

        $this->postData = $this->input->post();
        $this->tb = otable::sceneTriggerConfig();
    }

    /**
     * 配置列表
     */
    public function index()
    {
        $data = [];
        $data['sceneList'] = self::ALL_SCENE;
        $data['coolTime'] = oo::commonOprModel('config')->getCon("Game", "sceneTriggerCoolTime");
        $data['list'] = $this->formatTriggerConf();

        $this->load->view("sceneTrigger/index", $data);
    }

    /**
     * 配置上下线变动
     */
    public function updateSwitch()
    {
        $id = $this->postData['id'];
        $switch = $this->postData['switch'];

        $ret = oo::commonOprDb('games')->query("Update {$this->tb} set  `switch`='{$switch}'  where id={$id}");

        if (!$ret) {
            return $this->response(1, '操作失败');
        } else {
            return $this->response(0, '操作成功');
        }
    }

    public function delScene()
    {
        $id = $this->postData['id'];
        $ret = SceneTriggerModel::delById($id);

        if (!$ret) {
            return $this->response(1, '操作失败');
        } else {
            return $this->response(0, '操作成功');
        }
    }

    /**
     * 触发奖励配置变动
     */
    public function updateConfig()
    {
        $formData = $this->postData;
        $dateRange = explode(' - ', $formData['sn_date']);
        unset($formData['sn_date']);

        $formData['date_start'] = strtotime($dateRange[0]);
        $formData['date_end'] = strtotime($dateRange[1]);

        $ret = SceneTriggerModel::updateConfig($formData);

        if ($ret[0] != 0) {
            return $this->response(-1, $ret[1]);
        } else {
            return $this->response(0, '操作成功');
        }
    }

    public function updateCommon()
    {
        $coolTime = $this->postData['coolTime'];

        $ret = oo::commonOprModel('config')->updateCon("Game", "sceneTriggerCoolTime", $coolTime, '跨场景触发冷却时间(h)');

        // 更新json
        ConfigModel::updateConJson();

        if (!$ret) {
            return $this->response(1, '操作失败');
        } else {
            return $this->response(0, '操作成功');
        }
    }

    public function formatTriggerConf(): array
    {
        $sql = "SELECT * from {$this->tb} WHERE 1";
        $triggerConf = oo::commonOprDb('slavedb')->getAll($sql);

        $curTime = time();
        foreach ($triggerConf as &$item) {
            $item['date_start'] = date('Y-m-d H:i:s', $item['date_start']);
            $item['date_end'] = date('Y-m-d H:i:s', $item['date_end']);

            $item['sn_date'] = $item['date_start'] . " - " . $item['date_end'];
            $item['sceneHtml'] = $this->buildSceneHtml($item['scene']);

            if ($curTime < strtotime($item['date_start'])) {
                $item['status'] = 0; // 未开始
            } elseif ($curTime > strtotime($item['date_end'])) {
                $item['status'] = 2; // 已过期
            } else {
                $item['status'] = 1; // 进行中
            }
        }
        return $triggerConf;
    }

    public function buildSceneHtml($scene): string
    {
        $str = self::ALL_SCENE[$scene];
        return "<span style='margin-top: 2px;height: 30px;font-size: 12px;background: #337ecc' class='btn btn-primary'>$str</span>";
    }

}
