<?php

class CollectActCouponActModel extends ActBaseModel implements ActInterface
{

    function formatActData(array &$actConfig)
    {
        list("baseConf" => $baseConf, "stageConf" => $stageConf) = $actConfig['collectActCouponConf'];
        unset($actConfig['collectActCouponConf']);

        $actConfig['reward_conf'] = $baseConf;
        $actConfig['extraData'] = json_encode($stageConf, JSON_NUMERIC_CHECK);
    }

    function writeActCache(array $actConfig)
    {
        $rewardInfo = json_decode($actConfig['extraData'], true);
        $rewardConf = json_decode($actConfig['reward_conf'], true);

        $conditionMap = [];
        $stageRewardMap = [];
        foreach ($rewardInfo as $idx1 => $item1) {
            foreach ($item1['stages'] as $idx2 => $item2) {
                foreach ($item2['stages'] as $idx3 => $item3) {
                    $stageKey = "{$idx1}_{$idx2}_{$idx3}";
                    $conditionMap[$stageKey] = json_encode($this->getStageConditionLvYShushu($item1, $item2, $item3), JSON_NUMERIC_CHECK);
                    $stageRewardMap[$stageKey] = json_encode($item3['stages'], JSON_NUMERIC_CHECK);
                }
            }
        }

        $SpinsRatioKey = okeys::treasureSpinsRatio($actConfig['aid']);
        oo::commonOprRedis('games')->set($SpinsRatioKey, $rewardConf['spinsRatio']);

        $conditionKeys = okeys::activityConditionKeys($actConfig['aid']);
        $stageRewardKeys = okeys::activityRewardsStageKeys($actConfig['aid']);

        oo::commonOprRedis('games')->hMset($conditionKeys, $conditionMap);
        oo::commonOprRedis('games')->hMset($stageRewardKeys, $stageRewardMap);

        $expireTime = ActBaseModel::getActCacheExpireTime($actConfig['end_time']);#redis最少一分钟
        oo::commonOprRedis('games')->expireAt($SpinsRatioKey, oo::redisRandomExpire($expireTime));
        oo::commonOprRedis('games')->expireAt($conditionKeys, oo::redisRandomExpire($expireTime));
        oo::commonOprRedis('games')->expireAt($stageRewardKeys, oo::redisRandomExpire($expireTime));
    }

    function serializeShowConf(array &$actConfig)
    {
    }

    function importActData(array &$basicConfig, $sheetArray)
    {
        if (count($sheetArray) < 6) {
            $this->response(-1 * __LINE__, "工作表数量不匹配");
        }

        $sheet1 = $sheetArray[0] ?? [];
        $sheet2 = $sheetArray[1] ?? [];
        $sheet3 = $sheetArray[2] ?? [];
        $sheet4 = $sheetArray[3] ?? [];
        $sheet5 = $sheetArray[4] ?? [];
        $sheet6 = $sheetArray[5] ?? [];

        $baseConf = $this->parseBaseSheet($sheet1);
        $stageConf = $this->parseLvYSsGroupStages($sheet2);
        $probConf = $this->parseProbSheet($sheet3);
        $outsideRewardConf = $this->parseRewardSheet($sheet4);
        $insideRewardConf = $this->parseRewardSheet($sheet5);
        $taskConf = $this->parseTaskSheet($sheet6);

        $basicConfig['collectActCouponConf'] = $this->packageConf($baseConf, $stageConf, $probConf, $outsideRewardConf, $insideRewardConf, $taskConf);
    }

    private function parseBaseSheet(array $sheet): array
    {
        $baseConfig = [
            "getScore" => 0,
            "spinsRatio" => 0,
        ];
        foreach ($sheet as $item) {
            $label = trim($item[0]);
            $value = intval($item[1]);
            switch ($label){
                case '积分基础值':
                    $baseConfig['getScore'] = $value;
                    break;
                case '体力系数':
                    $baseConfig['spinsRatio'] = $value;
                    break;
                default:

            }
        }

        return $baseConfig;
    }

    private function parseProbSheet(array $sheet): array
    {
        $confMap = [];
        for ($i = 1; $i < count($sheet); $i++) {
            $row = array_filter($sheet[$i]);
            if (empty($row)) continue;

            list($id, $seq, $bet, $costSpin, $prob) = $row;

            $confMap[$id][] = [
                "bet" => $bet,
                "costSpin" => $costSpin,
                "prob" => $prob
            ];
        }

        return $confMap;
    }

    private function parseStageSheet(array $sheet): array
    {
        $layer = [];
        for ($i = 1; $i < count($sheet); $i++) {
            list($id, $start, $end, $limit) = $sheet[$i];

            $layer[] = ["id" => $id, "start" => $start, "end" => $end, "min_scores" => $limit];
        }

        return $layer;
    }

    private function parseRewardSheet(array $sheet): array
    {
        $confMap = [];
        for ($i = 1; $i < count($sheet); $i++) {
            $row = array_filter($sheet[$i]);
            if (empty($row)) continue;

            $line = $i + 1;

            list($id, $stageId, $dst) = $sheet[$i];
            $prize = $this->parseLineRewards(array_slice($sheet[$i], 3), $line);

            $confMap[$id][] = [
                "stageId" => $stageId,
                "dst" => $dst,
                "prize" => $prize
            ];
        }

        return $confMap;
    }

    private function parseTaskSheet(array $sheet): array
    {
        $taskTypeMap = ["完成一关" => 1, "到达奖励关" => 2, "开出障碍物" => 3, "复活一次" => 4, "通关一次" => 5];
        $confMap = [];
        for ($i = 1; $i < count($sheet); $i++) {
            $row = array_filter($sheet[$i]);
            if (empty($row)) continue;

            list($id, $taskId, $taskTypeStr, $score) = $sheet[$i];
            $taskType = $taskTypeMap[$taskTypeStr] ?? 0;
            if (empty($taskType)) {
                $this->response(-1 * __LINE__, "唯一ID:{$id}, 任务类型:{$taskTypeStr} 不存在");
            }

            $confMap[$id][] = [
                "taskId" => count($confMap[$id]) + 1,
                "taskType" => $taskType,
                "score" => $score
            ];
        }

        return $confMap;
    }

    private function parseBigRewardSheet(array $sheet): array
    {
        $rewardArrMap = [];
        for ($i = 1; $i < count($sheet); $i++) {
            $line = $i + 1;

            $oneStage = ["club_target" => 0];

            list($id) = $sheet[$i];
            $oneStage['prizes'] = $this->parseLineRewards(array_slice($sheet[$i], 1), $line);

            $rewardArrMap[$id] = $oneStage;
        }

        return $rewardArrMap;
    }

    private function packageConf(array $baseConf, array $stageConf, array $probConf, array $outsideRewardConf, array $insideRewardConf, array $taskConf): array
    {
        $conf = ["baseConf" => $baseConf];
        foreach ($stageConf as &$item) {
            foreach ($item['stages'] as &$item2) {
                foreach ($item2['stages'] as &$item3) {

                    $id = $item3['id'];
                    unset($item3['id']);

                    $item3['stages']['betConf'] = $probConf[$id] ?? [];
                    if (empty($item3['stages']['betConf'])) {
                        $this->response(-1 * __LINE__, "唯一ID:{$id}, 概率配置不存在");
                    }

                    $item3['stages']['stageConfOutside'] = $outsideRewardConf[$id] ?? [];
                    if (empty($item3['stages']['stageConfOutside'])) {
                        $this->response(-1 * __LINE__, "唯一ID:{$id}, 顶部进度条配置不存在");
                    }

                    $item3['stages']['stageConfInside'] = $insideRewardConf[$id] ?? [];
                    if (empty($item3['stages']['stageConfInside'])) {
                        $this->response(-1 * __LINE__, "唯一ID:{$id}, 底部进度条配置不存在");
                    }

                    $item3['stages']['taskConf'] = $taskConf[$id] ?? [];
                    if (empty($item3['stages']['taskConf'])) {
                        $this->response(-1 * __LINE__, "唯一ID:{$id}, 四选一任务配置不存在");
                    }
                }
            }
        }
        $conf['stageConf'] = $stageConf;

        return $conf;
    }

}