<?php

class HiddenTreasuresActModel extends ActBaseModel implements ActInterface
{
    static $prizeTypeCollect2 = 2;
    static $prizeTypeHighStage = 3;
    static $prizeTypeToken = 4;
    static $prizeTypeGhost = 5;
    static $configMap = [ // 总格子数=》 有效步数
        9 =>  [6, 7],
        16 => [12, 13, 14],
        25 => [20, 21, 22],
        36 => [30, 31, 32, 33],
    ];
    public $extraConfigFieldMapping = [
        '积分基础值' => 'getScore',
    ];
    function formatActData(array &$actConfig)
    {

        if(empty($actConfig['hiddenTreasuresConf']['baseConf']['getScore']))$this->response(1, '操作失败，积分基础值为空', $actConfig);
        if(empty($actConfig['hiddenTreasuresConf']['stageConf']))$this->response(1, '操作失败，配置为空', $actConfig);

        $noEmtypFieldsMap = [
            'betConf' => '概率',
            'maps' => '宝藏配置',
            'tokenRewardConf' => '积分获取代币配置',
        ];

        $gridField = [
            'mapSize' => '格子数',
            'steps' => '有效步数',
            'prize' => '奖励配置',
        ];
        foreach ($actConfig['hiddenTreasuresConf']['stageConf'] as $lvIdx => &$itemLv) {
            if (empty($itemLv['stages']))$this->response(__LINE__, "保存失败，第{$lvIdx}层等级层，配置为空");

            foreach ($itemLv['stages'] as $spinsIdx => &$itemSpins) {
                if (empty($itemSpins['stages']))$this->response(__LINE__, "保存失败，第{$lvIdx}层等级层，第{$spinsIdx}层体力层，配置为空");

                foreach ($itemSpins['stages'] as $ssIdx => &$itemSs) {
                    foreach ($noEmtypFieldsMap as $field => $text){
                        if (empty($itemSs[$field]))$this->response(__LINE__, "保存失败，第{$lvIdx}层等级层，第{$spinsIdx}层体力层，第{$ssIdx}层数数层，{$text} 配置为空");
                    }

                    foreach ($itemSs['maps'] as $gridIdx => &$grid){
                        $gridKey = $gridIdx + 1;
                        foreach ($gridField as $field => $text){
                            if (empty($grid[$field])) $this->response(__LINE__, "保存失败，第{$lvIdx}层等级层，第{$spinsIdx}层体力层，第{$ssIdx}层数数层，第{$gridKey}层配置，{$text}为空", $grid);
                        }
                    }
                }
            }
        }
        $actConfig['reward_conf'] = $actConfig['hiddenTreasuresConf'];
        $actConfig['extraData'] = $actConfig['hiddenTreasuresConf']['baseConf'];
        unset($actConfig['hiddenTreasuresConf']);
        $this->saveTokenGoods($actConfig, 'pro');
    }

    function writeActCache(array $actConfig)
    {
        $conditionKeys = okeys::activityConditionKeys($actConfig['aid']);
        $stageRewardKeys = okeys::activityRewardsStageKeys($actConfig['aid']);
        $stageExtraConfKeys = okeys::activityStageExtraConfigKeys($actConfig['aid']);
        $rewardConf = json_decode($actConfig['reward_conf'], true);
        $extraReward = json_decode($actConfig['reward_pro'], true);

        foreach ($rewardConf['stageConf'] as $idx1 => $item1) {
            foreach ($item1['stages'] as $idx2 => $item2) {
                foreach ($item2['stages'] as $idx3 => $item3) {
                    $idx = "{$idx1}_{$idx2}_{$idx3}";
                    $condition = $this->getStageConditionLvSpinsShushu($item1, $item2, $item3);
                    oo::commonOprRedis('games')->hset($stageRewardKeys, $idx, json_encode($item3, 32));
                    oo::commonOprRedis('games')->hset($conditionKeys, $idx, json_encode($condition, 32));
                    oo::commonOprRedis('games')->hset($stageExtraConfKeys, $idx, json_encode(['tokenGoods' =>$extraReward[$idx] ?? []], 32));
                }
            }
        }
        $expireTime = ActBaseModel::getActCacheExpireTime($actConfig['end_time']);#redis最少一分钟
        oo::commonOprRedis('games')->expireAt($conditionKeys, oo::redisRandomExpire($expireTime));
        oo::commonOprRedis('games')->expireAt($stageRewardKeys, oo::redisRandomExpire($expireTime));
        oo::commonOprRedis('games')->expireAt($stageExtraConfKeys, oo::redisRandomExpire($expireTime));
        return true;
    }

    function serializeShowConf(array &$actConfig)
    {
        // TODO: Implement serializeShowConf() method.
    }

    function importActData(array &$basicConfig, $sheetArray)
    {
        $sheetIdx = -1;
        $basicArray = $sheetArray[++$sheetIdx] ?? [];
        $extraCfgSheet = $sheetArray[++$sheetIdx] ?? [];    // 额外配置
        $stagesMappingSheet = $sheetArray[++$sheetIdx] ?? [];   // 分层关系配置
        $probSheet = $sheetArray[++$sheetIdx] ?? [];      //概率配置
        $scoreToTokensSheet = $sheetArray[++$sheetIdx] ?? [];      //积分获取代币配置
        $rewardsSheet = $sheetArray[++$sheetIdx] ?? [];      //宝藏配置
        $noTokenSheet = $sheetArray[++$sheetIdx] ?? [];      //代币不足配置

        list($rewardsConfig, $idxMapping) = $this->parseLvSpinsShushuStagesMapping($stagesMappingSheet);

        // 解析概率
        $this->parseProbSheet($rewardsConfig, $idxMapping, $probSheet);

        $this->parseScoreToTokenSheet($rewardsConfig, $idxMapping, $scoreToTokensSheet);

        $this->parseRewardSheet($rewardsConfig, $idxMapping, $rewardsSheet);

        $this->formatTokenGoodsSheet($basicConfig, $noTokenSheet, $idxMapping);

        $basicConfig['hiddenTreasuresConf'] = [
            'stageConf' => $rewardsConfig,
            'baseConf' => self::parseExtraBasicConfig($this->extraConfigFieldMapping, $extraCfgSheet)
        ];
    }


    // 概率配置
    public function parseProbSheet(&$rewardsConfig, $idxMapping, $sheetData){
        $sheetCnt = count($sheetData);
        for($i = 1;$i < $sheetCnt;$i++){
            list($idx, $bet, $spins, $prob) = $sheetData[$i];
            $lineNum = $i+1;
            if (empty($idxMapping[$idx])) $this->response(__LINE__, "第{$lineNum}行,唯一id:{$idx} 无对应关系", $sheetData[$i]);
            list($lvIdx, $spinsIdx, $ssIdx) = $idxMapping[$idx];
            $rewardsConfig[$lvIdx]['stages'][$spinsIdx]['stages'][$ssIdx]['betConf'] = $rewardsConfig[$lvIdx]['stages'][$spinsIdx]['stages'][$ssIdx]['betConf'] ?? [];
            $rewardsConfig[$lvIdx]['stages'][$spinsIdx]['stages'][$ssIdx]['betConf'][] = [
                'bet' => $bet,
                'costSpin' => $spins,
                'prob' => $prob,
            ];
        }
    }

    // 入场积分配置
    public function parseScoreToTokenSheet(&$rewardsConfig, $idxMapping, $sheetData){
        $sheetCnt = count($sheetData);
        for($i = 1;$i < $sheetCnt;$i++){
            list($idx, $score, $tokens) = $sheetData[$i];
            $lineNum = $i+1;
            if (empty($idxMapping[$idx])) $this->response(__LINE__, "第{$lineNum}行,唯一id:{$idx} 无对应关系", $sheetData[$i]);
            list($lvIdx, $spinsIdx, $ssIdx) = $idxMapping[$idx];
            $rewardsConfig[$lvIdx]['stages'][$spinsIdx]['stages'][$ssIdx]['tokenRewardConf'] = $rewardsConfig[$lvIdx]['stages'][$spinsIdx]['stages'][$ssIdx]['tokenRewardConf'] ?? [];
            $stageCnt = count($rewardsConfig[$lvIdx]['stages'][$spinsIdx]['stages'][$ssIdx]['tokenRewardConf']);
            $rewardsConfig[$lvIdx]['stages'][$spinsIdx]['stages'][$ssIdx]['tokenRewardConf'][] = [
                'stageId' => $stageCnt + 1 ,
                'dst' => $score,
                'tokens' => $tokens,
            ];
        }
    }

    // 宝藏sheet
    public function parseRewardSheet(&$rewardsConfig, $idxMapping, $sheetData){
        $sheetCnt = count($sheetData);
        for($i = 1;$i < $sheetCnt;$i++){
            list($idx, $gridType, $steps) = $sheetData[$i];
            $lineNum = $i+1;
            if (empty($idxMapping[$idx])) $this->response(__LINE__, "第{$lineNum}行,唯一id:{$idx} 无对应关系", $sheetData[$i]);

            if( empty(self::$configMap[$gridType]))$this->response(__LINE__, "第{$lineNum}行,格子数异常：{$gridType}", $sheetData[$i]);

            if( !in_array($steps, self::$configMap[$gridType]))$this->response(__LINE__, "第{$lineNum}行,有效步数异常：{$steps}", $sheetData[$i]);

            list($lvIdx, $spinsIdx, $ssIdx) = $idxMapping[$idx];
            $rewardsConfig[$lvIdx]['stages'][$spinsIdx]['stages'][$ssIdx]['maps'] = $rewardsConfig[$lvIdx]['stages'][$spinsIdx]['stages'][$ssIdx]['maps'] ?? [];

            $prizes = $this->parseLineRewards( array_slice($sheetData[$i], 3), $lineNum);

            $rewardsConfig[$lvIdx]['stages'][$spinsIdx]['stages'][$ssIdx]['maps'][] = [
                "stage" => 1 + count($rewardsConfig[$lvIdx]['stages'][$spinsIdx]['stages'][$ssIdx]['maps']),
                "mapSize" => $gridType,
                "steps" => $steps,
                'prize' => $prizes
            ];
        }
    }


    public function getActBasicConfig(): array
    {
        return self::$configMap;
    }
}