<?php

class SuperGashaponActModel extends ActBaseModel implements ActInterface
{

    function formatActData(array &$actConfig)
    {

        if (empty($actConfig['superGashaponConfig'])) {
            $this->response(1, '保存失败，未配置等级层级');
        }

        # 处理奖励
        foreach ($actConfig['superGashaponConfig'] as $lvIdx => &$lvItem) {
            if (empty($lvItem['stages'])) $this->response(-1 * __LINE__, "等级层级: {$lvIdx} ,未配置体力层级");

            foreach ($lvItem['stages'] as $spinsIdx => &$spinsItem) {

                if (empty($spinsItem['stages'])) $this->response(-1 * __LINE__, "等级层级: {$lvIdx} 体力层级: {$spinsIdx} ,未配置奖励层级");

                foreach ($spinsItem['stages'] as $ssIdx => &$ssItem) {
                    if (empty($ssItem['reward'])) $this->response(-1 * __LINE__, "等级层级: {$lvIdx} 体力层级: {$spinsIdx} 数数层级: {$ssIdx} ,未配置奖励层级");

                    $prizeStat = [];
                    foreach ($ssItem['reward'] as $rewardIdx => &$rewardItem) {
                        if (!isset($rewardItem['prize']))$rewardItem['prize'] = [];

                        if ($rewardItem['gashType'] != 2) {
                            if (count($rewardItem['prize']) != 1)
                                $this->response(-1 * __LINE__, "等级层级: {$lvIdx} 体力层级: {$spinsIdx} 数数层级: {$ssIdx} 奖励层级: {$rewardIdx} ,奖励必须配置且唯一");
                        } else {
                            if (!empty($rewardItem['prize']))
                                $this->response(-1 * __LINE__, "等级层级: {$lvIdx} 体力层级: {$spinsIdx} 数数层级: {$ssIdx} 奖励层级: {$rewardIdx} ,幸运蛋不能配置奖励");
                        }

                        $prizeStat[$rewardItem['layer']][] = $rewardItem['gashType'] == 2 ? 1 : 0;
                    }

                    foreach ($prizeStat as $layer => $prizeInfo) {
                        // 幸运蛋最多只能有一个
                        if (array_sum($prizeInfo) > 1) $this->response(-1 * __LINE__, "等级层级: {$lvIdx} 体力层级: {$spinsIdx} 扭蛋层级: {$layer} ,每层幸运蛋最多一个");

                        // 总奖励数最多10个
                        if (count($prizeInfo) > 10)$this->response(-1 * __LINE__, "等级层级: {$lvIdx} 体力层级: {$spinsIdx} 扭蛋层级: {$layer} ,每层扭蛋不能超10个");
                    }
                }
            }
        }

        $actConfig['reward_info'] = json_encode($actConfig['superGashaponConfig'], JSON_NUMERIC_CHECK);
        unset($actConfig['superGashaponConfig']);

        $this->saveTokenGoods($actConfig, 'extraData');
    }

    function writeActCache(array $actConfig)
    {
        $conditionKeys = okeys::activityConditionKeys($actConfig['aid']);
        $stageRewardKeys = okeys::activityRewardsStageKeys($actConfig['aid']);
        $stageExtraConfKeys = okeys::activityStageExtraConfigKeys($actConfig['aid']);
        $rewardInfo = json_decode($actConfig['reward_info'], true);
        $extraData = json_decode($actConfig['extraData'], true);
        foreach ($rewardInfo as $idx1 => $item1) {
            foreach ($item1['stages'] as $idx2 => $item2) {
                foreach ($item2['stages'] as $idx3 => $item3) {
                    $stageKey = "{$idx1}_{$idx2}_{$idx3}";
                    $stageConditions = $this->getStageConditionLvSpinsShushu($item1, $item2, $item3);
                    oo::commonOprRedis('games')->hset($conditionKeys, $stageKey, json_encode($stageConditions, JSON_NUMERIC_CHECK));
                    oo::commonOprRedis('games')->hset($stageRewardKeys, $stageKey, json_encode($item3['reward'], JSON_NUMERIC_CHECK));
                    oo::commonOprRedis('games')->hset($stageExtraConfKeys, $stageKey, json_encode(['tokenGoods' =>$extraData[$stageKey]], JSON_NUMERIC_CHECK));
                }
            }
        }

        $expireTime = ActBaseModel::getActCacheExpireTime($actConfig['end_time']);#redis最少一分钟
        oo::commonOprRedis('games')->expireAt($conditionKeys, oo::redisRandomExpire($expireTime));
        oo::commonOprRedis('games')->expireAt($stageRewardKeys, oo::redisRandomExpire($expireTime));
        oo::commonOprRedis('games')->expireAt($stageExtraConfKeys, oo::redisRandomExpire($expireTime));
    }

    function serializeShowConf(array &$actConfig)
    {
        $rewardString = $actConfig['reward_info'];
        $rewardInfo = json_decode($rewardString, true);
        // unset 每个区间的奖励配置 用ajax异步加载
        foreach ($rewardInfo as &$lvItem) {
            foreach ($lvItem['stages'] as &$spinsItem) {
                foreach ($spinsItem['stages'] as &$ssItem) {
                    $ssItem['reward'] = [];
                }
            }
        }

        $actConfig['reward_info'] = json_encode($rewardInfo);
    }

    function importActData(array &$basicConfig, $sheetArray)
    {
        $gashTypeMap = ["普通" => 0, "大奖" => 1, "幸运蛋" => 2];
        $sheetIdx = 0;
        $stagesMappingArray = $sheetArray[++$sheetIdx] ?? [];
        $stagesRewardsArray = $sheetArray[++$sheetIdx] ?? [];
        $noTokenConfigArray = $sheetArray[++$sheetIdx] ?? [];
        list($rewardsConfig, $idxMapping) = $this->parseLvSpinsShushuStagesMapping($stagesMappingArray);

        foreach ($stagesRewardsArray as $k => $item){
            if(empty($k))continue;
            if (empty(array_filter($item))) continue;
            $line = $k + 1;
            @list($idx, $layer, $gashTypeText, $weight, $needCoins) = $item;
            @list($lvStage, $spinsStage, $ssStage) = $idxMapping[$idx];

            $gashType = $gashTypeMap[$gashTypeText] ?? null;
            if ($gashType === null) {
                $this->response(-1, "第{$line}行,扭蛋类型异常:".$gashTypeText);
            }
            $rewardStage = count($rewardsConfig[$lvStage]['stages'][$spinsStage]['stages'][$ssStage]['reward']) + 1;

            $rewardsTemp = [];
            if ($gashType != 2) {
                $rewardsTemp = $this->parseLineRewards( array_slice($item, 5), $line);
                if (empty($rewardsTemp)) {
                    $this->response(-1, "第{$line}行,唯一id:{$idx} 无奖励配置");
                }
            }

            $rewardsConfig[$lvStage]['stages'][$spinsStage]['stages'][$ssStage]['reward'][$rewardStage] = [
                'layer' => $layer,
                'gashType' => $gashType,
                'weight' => $weight,
                'needCoins' => $needCoins,
                'prize' => $rewardsTemp,
            ];

        }

        $this->formatTokenGoodsSheet($basicConfig, $noTokenConfigArray, $idxMapping);
        $basicConfig['superGashaponConfig'] = $rewardsConfig;
    }
}