<?php

class SurpriseDiceActModel extends ActBaseModel implements ActInterface
{
    static $prizeTypeCollect1 = 1;
    static $prizeTypeCollect2 = 2;
    static $prizeTypeHighStage = 3;
    static $prizeTypeToken = 4;
    static $prizeTypeGhost = 5;
    static $prizeTypes = [
        1 => "收集物1",
        2 => "收集物2",
        3 => "收集物3",
        4 => "收集物4",
        5 => "过关点",
        6 => "退场点",
        99 => "万能符",
    ];
    public $extraConfigFieldMapping = [
        '积分基础值' => 'getScore',
    ];
    function formatActData(array &$actConfig)
    {

        if(empty($actConfig['surpriseDiceConf']['baseConf']['getScore']))$this->response(1, '操作失败，积分基础值为空', $actConfig);
        if(empty($actConfig['surpriseDiceConf']['stageConf']))$this->response(1, '操作失败，配置为空', $actConfig);


        $noEmtypFieldsMap = [
            'giveToken' => '入场赠送代币',
            'betConf' => '概率',
            'joinStageConf' => '入场门槛积分',
            'layerConf' => '关卡',
//            'collectConf' => '进度条',
        ];
        $collectIdxes = [0,1,2,3];
        $progressMax = 6;
        foreach ($actConfig['surpriseDiceConf']['stageConf'] as $lvIdx => &$itemLv) {
            if (empty($itemLv['stages']))$this->response(__LINE__, "保存失败，第{$lvIdx}层等级层，配置为空");

            foreach ($itemLv['stages'] as $spinsIdx => &$itemSpins) {
                if (empty($itemSpins['stages']))$this->response(__LINE__, "保存失败，第{$lvIdx}层等级层，第{$spinsIdx}层体力层，配置为空");

                foreach ($itemSpins['stages'] as $ssIdx => &$itemSs) {
                    // 非空字段校验
                    foreach ($noEmtypFieldsMap as $field => $text){
                        if (empty($itemSs[$field]))$this->response(__LINE__, "保存失败，第{$lvIdx}层等级层，第{$spinsIdx}层体力层，第{$ssIdx}层数数层，{$text} 配置为空");
                    }
                    // 骰子权重配置校验
                    foreach ($itemSs['layerConf'] as $rewardIdx => &$rewardProItem){
                        $collectKey = $rewardIdx + 1;
                        $rewardProItem['goldPrize'] = @json_decode(urldecode($rewardProItem['goldPrize']),  true) ?? [];
                        if(empty($rewardProItem['goldPrize']))
                            $this->response(__LINE__, "保存失败，第{$lvIdx}层等级层，第{$spinsIdx}层体力层，第{$ssIdx}层数数层，关卡{$collectKey} 大奖奖励配置为空");
                    }

                    // 进度条校验
                    foreach ($collectIdxes as $collectIdx){
                        $collectKey = $collectIdx + 1;
                        if (empty($itemSs['collectConf'][$collectIdx]))$this->response(__LINE__, "保存失败，第{$lvIdx}层等级层，第{$spinsIdx}层体力层，第{$ssIdx}层数数层，进度条{$collectKey} 配置为空");


                        foreach ($itemSs['collectConf'][$collectIdx] as $collectRowIdx => &$collectItem){
                            $collectRowKey = $collectRowIdx + 1;
                            if(empty($collectItem['dst']))
                                $this->response(__LINE__, "保存失败，第{$lvIdx}层等级层，第{$spinsIdx}层体力层，第{$ssIdx}层数数层，进度条{$collectKey} 第{$collectRowKey}层 阶段值配置为空");

                            $collectItem['prize'] = @json_decode(urldecode($collectItem['prize']), true) ?? [];
                            if(empty($collectItem['prize']))
                                $this->response(__LINE__, "保存失败，第{$lvIdx}层等级层，第{$spinsIdx}层体力层，第{$ssIdx}层数数层，进度条{$collectKey} 第{$collectRowKey}层 奖励配置为空");
                        }
                    }
                }
            }
        }

        $actConfig['reward_conf'] = $actConfig['surpriseDiceConf'];
        $actConfig['extraData'] = $actConfig['surpriseDiceConf']['baseConf'];
        unset($actConfig['surpriseDiceConf']);
        $this->saveTokenGoods($actConfig, 'pro');
    }

    function writeActCache(array $actConfig)
    {
        $conditionKeys = okeys::activityConditionKeys($actConfig['aid']);
        $stageRewardKeys = okeys::activityRewardsStageKeys($actConfig['aid']);
        $stageExtraConfKeys = okeys::activityStageExtraConfigKeys($actConfig['aid']);
        $rewardConf = json_decode($actConfig['reward_conf'], true);
        $extraReward = json_decode($actConfig['reward_pro'], true);

        foreach ($rewardConf['stageConf'] as $idx1 => $item1) {
            foreach ($item1['stages'] as $idx2 => $item2) {
                foreach ($item2['stages'] as $idx3 => $item3) {
                    $idx = "{$idx1}_{$idx2}_{$idx3}";
                    $condition = $this->getStageConditionLvSpinsShushu($item1, $item2, $item3);
                    oo::commonOprRedis('games')->hset($stageRewardKeys, $idx, json_encode($item3, 32));
                    oo::commonOprRedis('games')->hset($conditionKeys, $idx, json_encode($condition, 32));
                    oo::commonOprRedis('games')->hset($stageExtraConfKeys, $idx, json_encode(['tokenGoods' =>$extraReward[$idx] ?? []], 32));
                }
            }
        }
        $expireTime = ActBaseModel::getActCacheExpireTime($actConfig['end_time']);#redis最少一分钟
        oo::commonOprRedis('games')->expireAt($conditionKeys, oo::redisRandomExpire($expireTime));
        oo::commonOprRedis('games')->expireAt($stageRewardKeys, oo::redisRandomExpire($expireTime));
        oo::commonOprRedis('games')->expireAt($stageExtraConfKeys, oo::redisRandomExpire($expireTime));
        return true;
    }

    function serializeShowConf(array &$actConfig)
    {
        // TODO: Implement serializeShowConf() method.
    }

    function importActData(array &$basicConfig, $sheetArray)
    {
        $sheetIdx = -1;
        $basicArray = $sheetArray[++$sheetIdx] ?? [];
        $extraCfgSheet = $sheetArray[++$sheetIdx] ?? [];    // 额外配置
        $stagesMappingSheet = $sheetArray[++$sheetIdx] ?? [];   // 分层关系配置
        $giveTokenSheet = $sheetArray[++$sheetIdx] ?? [];        // 入场代币赠送配置
        $probSheet = $sheetArray[++$sheetIdx] ?? [];      //概率配置
        $joinScoreSheet = $sheetArray[++$sheetIdx] ?? [];      //入场积分配置
        $rewardProSheet = $sheetArray[++$sheetIdx] ?? [];      //普通配置
        $progress1Sheet = $sheetArray[++$sheetIdx] ?? [];      //进度条1配置
        $progress2Sheet = $sheetArray[++$sheetIdx] ?? [];      //进度条2配置
        $progress3Sheet = $sheetArray[++$sheetIdx] ?? [];      //进度条2配置
        $progress4Sheet = $sheetArray[++$sheetIdx] ?? [];      //进度条2配置
        $noTokenSheet = $sheetArray[++$sheetIdx] ?? [];      //代币不足配置

        list($rewardsConfig, $idxMapping) = $this->parseLvSpinsShushuStagesMapping($stagesMappingSheet);

        $this->parseFreeToken($rewardsConfig, $idxMapping, $giveTokenSheet);

        $this->parseProbSheet($rewardsConfig, $idxMapping, $probSheet);

        $this->parseJoinScoreSheet($rewardsConfig, $idxMapping, $joinScoreSheet);

        $this->parseReewardProSheet($rewardsConfig, $idxMapping, $rewardProSheet);

        $collectIdxes = [0,1,2,3];
        foreach ($collectIdxes as $collectIdx){
            $collectKey = $collectIdx + 1;
            $sheetName = "progress{$collectKey}Sheet";
            $this->parseProgressSheet($rewardsConfig, $idxMapping, $$sheetName, $collectIdx);
        }

        $this->formatTokenGoodsSheet($basicConfig, $noTokenSheet, $idxMapping);

        $basicConfig['surpriseDiceConf'] = [
            'stageConf' => $rewardsConfig,
            'baseConf' => self::parseExtraBasicConfig($this->extraConfigFieldMapping, $extraCfgSheet)
        ];
    }

    // 入场赠送代币配置
    public function parseFreeToken(&$rewardsConfig, $idxMapping, $sheetData){
        $sheetCnt = count($sheetData);
        for($i = 1;$i < $sheetCnt;$i++){
            list($idx, $tokens) = $sheetData[$i];
            $lineNum = $i+1;
            if (empty($idxMapping[$idx])) $this->response(__LINE__, "第{$lineNum}行,唯一id:{$idx} 无对应关系", [$sheetData[$i], $idxMapping]);
            list($lvIdx, $spinsIdx, $ssIdx) = $idxMapping[$idx];
            $rewardsConfig[$lvIdx]['stages'][$spinsIdx]['stages'][$ssIdx]['giveToken'] = $tokens;
        }
    }

    // 概率配置
    public function parseProbSheet(&$rewardsConfig, $idxMapping, $sheetData){
        $sheetCnt = count($sheetData);
        for($i = 1;$i < $sheetCnt;$i++){
            list($idx, $bet, $spins, $prob) = $sheetData[$i];
            $lineNum = $i+1;
            if (empty($idxMapping[$idx])) $this->response(__LINE__, "第{$lineNum}行,唯一id:{$idx} 无对应关系", $sheetData[$i]);
            list($lvIdx, $spinsIdx, $ssIdx) = $idxMapping[$idx];
            $rewardsConfig[$lvIdx]['stages'][$spinsIdx]['stages'][$ssIdx]['betConf'] = $rewardsConfig[$lvIdx]['stages'][$spinsIdx]['stages'][$ssIdx]['betConf'] ?? [];
            $rewardsConfig[$lvIdx]['stages'][$spinsIdx]['stages'][$ssIdx]['betConf'][] = [
                'bet' => $bet,
                'costSpin' => $spins,
                'prob' => $prob,
            ];
        }
    }

    // 入场积分配置
    public function parseJoinScoreSheet(&$rewardsConfig, $idxMapping, $sheetData){
        $sheetCnt = count($sheetData);
        for($i = 1;$i < $sheetCnt;$i++){
            list($idx, $score) = $sheetData[$i];
            $lineNum = $i+1;
            if (empty($idxMapping[$idx])) $this->response(__LINE__, "第{$lineNum}行,唯一id:{$idx} 无对应关系", $sheetData[$i]);
            list($lvIdx, $spinsIdx, $ssIdx) = $idxMapping[$idx];
            $rewardsConfig[$lvIdx]['stages'][$spinsIdx]['stages'][$ssIdx]['joinStageConf'] = $rewardsConfig[$lvIdx]['stages'][$spinsIdx]['stages'][$ssIdx]['joinStageConf'] ?? [];
            $stageCnt = count($rewardsConfig[$lvIdx]['stages'][$spinsIdx]['stages'][$ssIdx]['joinStageConf']);
            $rewardsConfig[$lvIdx]['stages'][$spinsIdx]['stages'][$ssIdx]['joinStageConf'][] = [
                'stageId' => $stageCnt + 1 ,
                'dst' => $score,
            ];
        }
    }

    // 关卡
    public function parseReewardProSheet(&$rewardsConfig, $idxMapping, $sheetData){
        $sheetCnt = count($sheetData);
        for($i = 1;$i < $sheetCnt;$i++){

            $idx = $sheetData[$i][0];
            $lineNum = $i+1;

            // 映射关系
            if (empty($idxMapping[$idx])) $this->response(__LINE__, "第{$lineNum}行,唯一id:{$idx} 无对应关系", $sheetData[$i]);
            list($lvIdx, $spinsIdx, $ssIdx) = $idxMapping[$idx];

            // 层级配置
            $rewardsConfig[$lvIdx]['stages'][$spinsIdx]['stages'][$ssIdx]['layerConf'] = $rewardsConfig[$lvIdx]['stages'][$spinsIdx]['stages'][$ssIdx]['layerConf'] ?? [];
            $layerId = count($rewardsConfig[$lvIdx]['stages'][$spinsIdx]['stages'][$ssIdx]['layerConf']) + 1;
            $layerConf = [
                'layerId' => $layerId,
                'weights' => [],
            ];

            // 骰子权重
            for($j = 0;$j < 3;$j++){
                $layerConf['weights'][$j] = [];
                $diceIdx = 0;
                foreach (self::$prizeTypes as $prizeIdx => $text){
                    $diceIdx++;
                    $columnIdx = $j * 7 + $diceIdx ;
                    $layerConf['weights'][$j][$prizeIdx] = intval($sheetData[$i][$columnIdx]);
                }
            }
            $prizeCostIdx = 3 * 7 + 1;
            $layerConf['passNeedCost'] = intval($sheetData[$i][$prizeCostIdx]);
            $layerConf['exitNeedCost'] = intval($sheetData[$i][$prizeCostIdx+1]);

            $prize = $this->parseLineRewards(array_slice($sheetData[$i], $prizeCostIdx + 2), $lineNum);
            $layerConf['goldPrize'] = urlencode(json_encode($prize, 32));

            $rewardsConfig[$lvIdx]['stages'][$spinsIdx]['stages'][$ssIdx]['layerConf'][] = $layerConf;
        }
    }

    // 进度条
    public function parseProgressSheet(&$rewardsConfig, $idxMapping, $sheetData,$progressType){
        $sheetCnt = count($sheetData);
        $collectIdxes = [0,1,2,3];
        for($i = 1;$i < $sheetCnt;$i++){
            list($idx, $dst) = $sheetData[$i];
            $lineNum = $i+1;
            if (!in_array($progressType, $collectIdxes)) $this->response(__LINE__, "操作异常，请联系开发人员", $sheetData[$i]);
            if (empty($idxMapping[$idx])) $this->response(__LINE__, "第{$lineNum}行,唯一id:{$idx} 无对应关系", $sheetData[$i]);
            list($lvIdx, $spinsIdx, $ssIdx) = $idxMapping[$idx];
            $rewardsConfig[$lvIdx]['stages'][$spinsIdx]['stages'][$ssIdx]['collectConf'][$progressType] = $rewardsConfig[$lvIdx]['stages'][$spinsIdx]['stages'][$ssIdx]['collectConf'][$progressType] ?? [];

            $prizes = $this->parseLineRewards( array_slice($sheetData[$i], 2), $lineNum);

            $rewardsConfig[$lvIdx]['stages'][$spinsIdx]['stages'][$ssIdx]['collectConf'][$progressType][] = [
                "stageId" => 1 + count($rewardsConfig[$lvIdx]['stages'][$spinsIdx]['stages'][$ssIdx]['collectConf'][$progressType]),
                "dst" => $dst,
                'prize' => urlencode(json_encode($prizes))
            ];
        }
    }


    public function getActBasicConfig(): array
    {
        return self::$prizeTypes;
    }
}