<?php

class NewShopModel extends CI_Model
{
    const IMG_NAME_SPRINT_PAY = 'sprintPay';            //冲刺礼包
    const IMG_NAME_FIRST_PAY = 'firstPay';              //首充礼包
    const IMG_NAME_UNFITNESS_PAY = 'unfitnessPay';      //体力不足礼包
    const IMG_NAME_UPGRADE_PAY = 'upgradePay';          //升级礼包

    private $giftPackGoodsTable;
    private $gTypeGoodsTable;
    private $propsConfig;
    private $ssGroupList;
    private $gTypeMap;
    private $payFixFields;
    private $imgSkinList;
    private $skuList;

    private $basicFieldsMap = [
        '唯一ID' => 'uuid',
        '触发等级' => 'min_level',
        '等级下限' => 'min_level',
        '等级上限' => 'max_level',
        '购买价格' => 'price',
        '数数分组' => 'ss_group',
        '配图' => 'skin_dir',
    ];

    private $extraFieldsMap = [
        '唯一ID' => 'uuid',
        '优惠显示' => 'showMultiple',
    ];

    const MultiNumTypeReward = ['随机金卡', '金币'];

    public function __construct()
    {
        parent::__construct();
        $this->giftPackGoodsTable = otable::giftPackGoods();
        $this->gTypeGoodsTable = otable::gTypeGoods();
        $this->propsConfig = oo::commonOprModel('config')->getPropsConfigFromLocal('func');
        $this->ssGroupList = array_column(oo::commonOprModel('config')->getUserGroupConfigByCid(5), 'name', 'id');
        $this->gTypeMap = self::getGTypeMap();
        $this->skuList = oo::cfg('sku');

        $this->load->library('PHPExcel');
        $this->load->library('PHPExcel/PHPExcel/IOFactory');
        $this->load->library('curl');
    }


    /**
     * 插入一条首充商品信息
     */
    public function insertFirstPayGoods(array $data, int $gType)
    {
        $formData = array(
            'gid' => 601,
            'g_type' => $gType,
            'desc' => '首充礼包商品',
            'sku' => $this->matchSKU($data['price']),
            'prize_json' => $this->formatPrize($data['props']['prizeJson'] ?? []),
            'other_json' => $this->formatPrize($data['other_json'] ?? []),
        );
        unset($data['props']);

        $this->insertGoods(array_merge($data, $formData));
    }

    /**
     * 更新首充商品信息
     */
    public function updateFirstPayGoods(array $data)
    {
        foreach ($data['good'] as $goodId => $oneGoods) {
            $oneGoods['sku'] = $this->matchSKU($oneGoods['price']);
            $oneGoods['prize_json'] = json_encode($oneGoods['prize_json'], JSON_NUMERIC_CHECK);
            $oneGoods['other_json'] = json_encode(["addRatio" => $oneGoods['addRatio']], JSON_NUMERIC_CHECK);
            unset($oneGoods['addRatio']);

            $this->updateGoods($oneGoods, "id=$goodId");
        }
    }

    /**
     * 更新体力不足商品信息
     */
    public function updateUnfitnessPayGoods(array $data)
    {
        $goodId = $data['id'];
        $data['life_time'] = 0;
        $data['skin_dir'] = '';
        $data['discount'] = 0;
        $data['sku'] = $this->matchSKU($data['price']);
        $data['ss_group'] = $this->formatSSGroup($data['ss_group']);
        $data['prize_json'] = json_encode($data['prize_json'] ?? [], JSON_NUMERIC_CHECK);

        unset($data['id'], $data['props'], $data['img_name']);

        $this->updateGoods($data, "id=$goodId");
    }

    /**
     * 更新冲刺商品信息
     */
    public function updateSprintPayGoods(array $data)
    {
        $goodId = $data['id'];
        $data['life_time'] = 0;
        $data['discount'] = 0;
        $data['sku'] = $this->matchSKU($data['price']);
        $data['ss_group'] = $this->formatSSGroup($data['ss_group']);
        $data['prize_json'] = json_encode($data['prize_json'] ?? [], JSON_NUMERIC_CHECK);

        unset($data['id'], $data['props'], $data['img_name']);

        $this->updateGoods($data, "id=$goodId");
    }

    /**
     * 更新升级商品信息
     */
    public function updateUpgradePayGoods(array $data)
    {
        $goodId = $data['id'];
        $data['max_level'] = $data['min_level'];
        $data['sku'] = $this->matchSKU($data['price']);
        $data['prize_json'] = json_encode($data['prize_json'] ?? [], JSON_NUMERIC_CHECK);
        $data['other_json'] = json_encode(['showMultiple' => $data['showMultiple']], JSON_NUMERIC_CHECK);

        //更新Json文件
        $this->updateUpgradeJson();

        $this->updateGoods($data, "id=$goodId");
    }

    /**
     * 更新礼包通用信息
     */
    public function updateGoodsCommonConf(array $data)
    {
        switch ($data['img_name']) {
            case self::IMG_NAME_SPRINT_PAY:
                oo::commonOprModel('config')->updateCon(self::IMG_NAME_SPRINT_PAY, "duration", $data['duration'], '冲刺礼包持续时间');   //持续时间
                oo::commonOprModel('config')->updateCon(self::IMG_NAME_SPRINT_PAY, "minLevel", $data['minLevel'], '冲刺礼包开放等级');   //持续时间
                break;
            case self::IMG_NAME_UPGRADE_PAY:
                oo::commonOprModel('config')->updateCon(self::IMG_NAME_UPGRADE_PAY, "duration", $data['duration'], '升级礼包持续时间');   //持续时间

                //更新Json文件
                $this->updateUpgradeJson();
                break;
        }
    }

    /**
     * 更新冲刺商品信息
     */
    public function importGoodsConfig($file, string $imgName): array
    {
        $this->imgSkinList = $this->getImgSkinList($imgName);

        //获取表格数据
        $dataArray = $this->getExcelSheetsArray($file);
        $basicConfArray = $dataArray[0] ?? [];
        $rewardsConfArray = $dataArray[1] ?? [];
        $extraConfArray = $dataArray[2] ?? [];

        //根据imgName取导入数据的字段配置
        self::createImportConfig($imgName);

        $gType = $this->gTypeMap[$imgName];

        $this->payFixFields['g_type'] = $gType;

        $basicConf = $this->formatExcelBasicConf($basicConfArray);
        if ($basicConf[0] === -1) {
            return $basicConf;
        }

        $rewardsConf = $this->formatExcelRewardConf($rewardsConfArray);
        if ($basicConf[0] === -1) {
            return $basicConf;
        }

        $extraConf = $this->formatExcelExtraConf($extraConfArray);

        $formatGoodsConf = $this->matchGoodsConf($basicConf, $rewardsConf, $extraConf);
        if ($formatGoodsConf[0] === -1) {
            return $formatGoodsConf;
        }

        //删除同一gtype的记录
        $sql = "DELETE FROM {$this->giftPackGoodsTable} WHERE g_type={$gType}";
        oo::commonOprDb('games')->query($sql);

        //插入新记录
        $sql = funs::db_intsertArrSQL($this->giftPackGoodsTable, $formatGoodsConf);
        oo::commonOprDb('games')->query($sql);

        //更新Json文件
        if (in_array($imgName, [self::IMG_NAME_UPGRADE_PAY])) {
            $this->updateUpgradeJson();
        }

        return [0, '操作成功'];
    }

    /**
     * 展示所有商品信息,通过gType
     * @param int $gType
     * @return array
     */
    public function showGoods(int $gType): array
    {
        $sql = "SELECT * FROM {$this->giftPackGoodsTable} where g_type={$gType};";
        $tableData = oo::commonOprDb('games')->getAll($sql, 1);

        foreach ($tableData as &$oneD) {
            $prizeJson = json_decode($oneD['prize_json'], true);
            $oneD['ss_group'] = json_decode($oneD['ss_group'], true);
            if (empty($oneD['ss_group']) || !is_array($oneD['ss_group'])) {
                $oneD['ssGroup'] = [];
            } else {
                foreach ($oneD['ss_group'] as $oneGroup) {
                    $oneD['ssGroup'][] = $this->ssGroupList[$oneGroup];
                }
            }
            $oneD = array_merge($oneD, json_decode($oneD['other_json'], true));

            $htmlStr = '<div>';
            foreach ($prizeJson as $onePrize) {
                $prizeStr = $this->matchPrizeInfo($onePrize);
                $htmlStr .= '<button style="margin-bottom: 5px" class="btn btn-outline btn-info btn-sm">' . $prizeStr . '</button><br>';
            }
            $htmlStr .= '</div>';
            $oneD['prizeJson'] = $htmlStr;
        }

        return $tableData;
    }

    /**
     * 展示通用配置信息
     * @param string $imgName
     * @return array
     */
    public function showCommonConf(string $imgName): array
    {
        switch ($imgName) {
            case self::IMG_NAME_SPRINT_PAY:
                $commonConf = [
                    "duration" => oo::commonOprModel('config')->getCon(self::IMG_NAME_SPRINT_PAY, "duration") ?? 0,   //持续时间
                    "minLevel" => oo::commonOprModel('config')->getCon(self::IMG_NAME_SPRINT_PAY, "minLevel") ?? 0,   //开放等级
                ];
                break;
            case self::IMG_NAME_UPGRADE_PAY:
                $commonConf = [
                    "duration" => oo::commonOprModel('config')->getCon(self::IMG_NAME_UPGRADE_PAY, "duration") ?? 0,   //持续时间
                ];
                break;
            default:
                $commonConf = [];

        }

        return $commonConf;
    }

    /**
     * 获取gType对应的view列表
     * @return array
     */
    public function getGTypeMap(): array
    {
        $sql = "select `g_type`, `en_name` FROM {$this->gTypeGoodsTable} where not isnull(`en_name`);";
        $gTypeArr = oo::commonOprDb('games')->getAll($sql, 1);

        $typeMap = [];
        foreach ($gTypeArr as $oneGType) {
            $typeMap[$oneGType['en_name']] = $oneGType['g_type'];
        }

        return $typeMap;
    }

    /**
     * 获取gType对应的view列表
     * @param string $imgName
     * @return array
     */
    public function getImgSkinList(string $imgName): array
    {
        if (in_array($imgName, [self::IMG_NAME_UNFITNESS_PAY, self::IMG_NAME_UPGRADE_PAY])) {
            return [];
        }

        $imgSkinList = oo::commonOprModel('config')->getActiveImgNameConfig($imgName)['content'] ?? [];

        $skinList = [];
        foreach ($imgSkinList as $oneSkin) {
            $skinList[$oneSkin['type']] = $oneSkin['name'];
        }

        return $skinList;
    }

    /**
     * 导出商品列表
     * @param string $imgName
     * @return PHPExcel
     * @throws PHPExcel_Exception
     */
    public function exportGoods(string $imgName): PHPExcel
    {
        $gType = $this->gTypeMap[$imgName];

        $sql = "SELECT * FROM {$this->giftPackGoodsTable} where g_type={$gType};";
        $tableData = oo::commonOprDb('slavedb')->getAll($sql);

        $objPHPExcel = new PHPExcel();
        $curActivitySheet = $objPHPExcel->setActiveSheetIndex(0);
        $curActivitySheet->setTitle('配置项(唯一ID用来匹配奖励项)');

        // 工作表1
        switch ($imgName) {
            case self::IMG_NAME_SPRINT_PAY:
                $headerTable = ['uuid' => '唯一ID', 'price' => '购买价格', 'min_level' => '等级下限', 'max_level' => '等级上限', 'ss_group' => '数数分组', 'skin_dir' => '配图'];
                break;
            case self::IMG_NAME_UPGRADE_PAY:
                $headerTable = ['uuid' => '唯一ID', 'min_level' => '触发等级', 'price' => '购买价格'];
                break;
            case self::IMG_NAME_UNFITNESS_PAY:
                $headerTable = ['uuid' => '唯一ID', 'price' => '购买价格', 'min_level' => '等级下限', 'max_level' => '等级上限', 'ss_group' => '数数分组'];
                break;
            default:
                return $objPHPExcel;
        }

        //处理表头
        $char = 'A';
        foreach ($headerTable as $k => $v) {
            $curActivitySheet->setCellValue($char . '1', $v);
            $char = chr(ord($char) + 1);
        }

        $maxPrizeLen = 0;
        $prizeList = [];
        $extraList = [];

        //处理数据
        $i = 2;
        foreach ($tableData as $vb) {
            $char = 'A';
            foreach ($headerTable as $hk => $hv) {
                if ($hk == 'uuid') {
                    $pValue = 1000 + $i;
                    $prizeInfo = json_decode($vb['prize_json'], true);
                    $maxPrizeLen = max(count($prizeInfo), $maxPrizeLen);
                    $prizeList[$pValue] = $prizeInfo;
                    $extraList[$pValue] = json_decode($vb['other_json'], true);
                } elseif ($hk == 'ss_group') {
                    $pValue = $this->ssGroup2String(json_decode($vb[$hk] ?? '[]', true));
                } else {
                    $pValue = $vb[$hk] ?? '';
                }

                $curActivitySheet->setCellValue($char . $i, $pValue);
                $char = chr(ord($char) + 1);
            }
            $i++;
        }

        // 工作表2
        $objPHPExcel->createSheet(1);
        $curActivitySheet = $objPHPExcel->setActiveSheetIndex(1);
        $curActivitySheet->setTitle('奖励项(支持任意项奖励)');

        $headerTable = ['propsId' => '奖励', 'rtype' => '是否固定值(非金币奖励均为固定值)', 'num' => '数值(宝箱等部分奖励数量固定为1)'];

        //处理表头
        $char = 'A';
        $curActivitySheet->setCellValue($char . '1', "唯一ID");
        $char = chr(ord($char) + 1);

        for ($i = 1; $i <= $maxPrizeLen; $i++) {
            foreach ($headerTable as $k => $v) {
                if ($k == 'propsId') {
                    $v .= $i;
                }
                $curActivitySheet->setCellValue($char . '1', $v);
                $char = chr(ord($char) + 1);
            }
        }

        //处理数据
        $i = 2;
        foreach ($prizeList as $uuid => $prizeArr) {
            $char = 'A';
            $curActivitySheet->setCellValue($char . $i, $uuid);
            $char = chr(ord($char) + 1);

            foreach ($prizeArr as $onePrize) {

                // 奖励名
                $rewardName = $this->getRewardNameByTypeAndSkinId($onePrize['propsId'], $onePrize['skin']);
                $curActivitySheet->setCellValue($char . $i, $rewardName);
                $char = chr(ord($char) + 1);

                // 奖励是否为固定
                $rtype = $onePrize['rtype'] == 1 ? '是' : '否';
                $curActivitySheet->setCellValue($char . $i, $rtype);
                $char = chr(ord($char) + 1);

                // 奖励数量
                $curActivitySheet->setCellValue($char . $i, $onePrize['num']);
                $char = chr(ord($char) + 1);
            }
            $i++;
        }


        if ($imgName == self::IMG_NAME_UPGRADE_PAY) {
            // 工作表3
            $objPHPExcel->createSheet(2);
            $curActivitySheet = $objPHPExcel->setActiveSheetIndex(2);
            $curActivitySheet->setTitle('额外字段(保存在JSON中)');

            $headerTable = ['uuid' => '唯一ID', 'showMultiple' => '优惠显示'];;

            //处理表头
            $char = 'A';
            foreach ($headerTable as $k => $v) {
                $curActivitySheet->setCellValue($char . '1', $v);
                $char = chr(ord($char) + 1);
            }

            //处理数据
            $i = 2;
            foreach ($extraList as $uuid => $extraArr) {
                $char = 'A';
                $curActivitySheet->setCellValue($char . $i, $uuid);
                $char = chr(ord($char) + 1);

                // 优惠显示
                $curActivitySheet->setCellValue($char . $i, $extraArr['showMultiple']);
                $i++;
            }
        }

        return $objPHPExcel;
    }

    /**
     * 数数分组转换成字符串类型
     * @param array $ssGroup
     * @return string
     */
    private function ssGroup2String(array $ssGroup): string
    {
        if (empty($ssGroup)) {
            return '';
        }

        $pValue = [];
        foreach ($ssGroup as $oneGroup) {
            $pValue[] = $this->ssGroupList[$oneGroup];
        }

        return implode(',', $pValue);
    }

    /**
     * 插入一条商品信息
     * @param array $data
     */
    private function insertGoods(array $data)
    {
        $this->filterFieldNull($this->giftPackGoodsTable, $data);
        $sql = funs::db_insertSQL($this->giftPackGoodsTable, $data);

        oo::commonOprDb('games')->query($sql);
    }

    /**
     * 更新商品信息
     * @param array $data
     * @param string $where
     */
    private function updateGoods(array $data, string $where)
    {
        $this->filterFieldNull($this->giftPackGoodsTable, $data);
        $sql = funs::db_updateSQL($this->giftPackGoodsTable, $data, $where);

        oo::commonOprDb('games')->query($sql);
    }


    /**
     * 匹配SKU，通过价格
     * @param float $price
     * @return string
     */
    private function matchSKU(float $price): string
    {
        //价格sku
        foreach ($this->skuList as $oneSkuInfo) {
            if ($oneSkuInfo['price'] == $price) {
                return $oneSkuInfo['sku'];
            }
        }

        return '';
    }

    private function formatPrize(array $prizeArr): string
    {
        $prizeArr = array_values($prizeArr);

        foreach ($prizeArr as &$onePrize) {
            foreach ($onePrize as $key => &$value) {
                $value = intval($value);
            }
        }

        return json_encode($prizeArr);
    }

    private function formatSSGroup(array $ssGroup): string
    {
        return json_encode($ssGroup, JSON_NUMERIC_CHECK);
    }

    private function filterFieldNull(string $tableName, array &$data)
    {
        $tableNameArr = explode('.', $tableName);
        $fieldsInfo = oo::commonOprDb('games')->getAll("select COLUMN_NAME from information_schema.COLUMNS
                    where table_name = '{$tableNameArr[1]}' and table_schema = '{$tableNameArr[0]}'");
        $fieldArr = array_column($fieldsInfo, 'COLUMN_NAME');

        foreach ($data as $key => &$value) {
            if ($value === null || !in_array($key, $fieldArr)) {
                unset($data[$key]);
            }
        }
    }

    private function getExcelSheetsArray($file): array
    {
        $xlsReader = IOFactory::createReader('Excel2007');
        $xlsReader->setReadDataOnly(true);
        $xlsReader->setLoadSheetsOnly(true);

        $Sheets = $xlsReader->load($file['url']);
        $sheetsCnt = $Sheets->getSheetCount();

        $dataArray = [];
        for ($i = 0; $i < $sheetsCnt; $i++) {
            $tmpData = $Sheets->getSheet($i)->toArray() ?? [];
            foreach ($tmpData as $k => $item) {
                foreach ($item as $k1 => $value) {
                    $item[$k1] = trim($value);
                }
                $line = array_filter($item);
                if (empty($line)) unset($tmpData[$k]);
            }
            $dataArray[$i] = $tmpData;
        }
        return $dataArray;
    }

    private function formatExcelBasicConf($basicConfArray): array
    {
        $ssGroupReverse = array_flip($this->ssGroupList);

        $fieldArr = [];

        foreach ($basicConfArray[0] as $field) {
            if (is_null($field)) {
                continue;
            }
            $fieldArr[] = $this->basicFieldsMap[$field];
        }

        $recordArr = [];
        for ($i = 1; $i < count($basicConfArray); $i++) {
            $oneRecord = [];
            foreach ($basicConfArray[$i] as $index => $item) {
                if (is_null($item)) {
                    continue;
                }
                $fieldName = $fieldArr[$index];
                switch ($fieldName) {
                    case "ss_group": // 数数分组
                        foreach (explode(',', $item) as $groupName) {
                            $ssIdx = $ssGroupReverse[$groupName] ?? null;
                            if (is_null($ssIdx)) {
                                return [-1, "数数分组 {$groupName} 不存在，检查数数分组是否配置"];
                            }
                            $oneRecord['ss_group'][] = intval($ssIdx);
                        }
                        break;
                    case "skin_dir": // 配图
                        if (empty($this->imgSkinList[$item])) {
                            return [-1, "配图资源 {$item} 不存在，检查资源是否配置"];
                        }
                        $oneRecord[$fieldName] = $item;
                        break;
                    case "price": // 价格
                        $sku = $this->matchSKU($item);
                        if (empty($sku)) {
                            return [-1, "价格 {$item} 不存在，检查是否配置错误"];
                        }
                        $oneRecord[$fieldName] = $item;
                        $oneRecord['sku'] = $sku;
                        break;
                    default:
                        $oneRecord[$fieldName] = $item;
                        break;
                }
            }
            $recordArr[] = $oneRecord;
        }

        return $recordArr;
    }

    /**
     *  解析奖励配置
     * @param array $stagesRewardsArray
     * @return array
     */
    private function formatExcelRewardConf(array $stagesRewardsArray): array
    {
        $rewardsConf = [];
        for ($line = 1; $line < count($stagesRewardsArray); $line++) {
            $item = $stagesRewardsArray[$line];

            $rewardsTemp = ['uuid' => $item[0], 'prize_json' => []];
            $rewardList = [];

            //分割奖励
            array_shift($item);
            for ($index = 0; $index < count($item); $index++) {
                $div = intval($index / 3);
                $mod = $index % 3;

                $value = $item[$index];
                if ($mod == 0) {
                    if (empty($value)) {
                        break;
                    }
                    $rewardList[$div]['rewardName'] = $value;

                } elseif ($mod == 1) {
                    $rewardList[$div]['isFixNum'] = $value;
                } else {
                    $rewardList[$div]['rewardNum'] = $value;
                }
            }

            //格式化奖励
            foreach ($rewardList as $oneReward) {
                list($propsId, $skin) = $this->parseRewardTypeAndSkinId($this->propsConfig, $oneReward['rewardName']);
                if (empty($propsId)) {
                    return [-1, "未找到奖励项：" . $oneReward['rewardName']];
                }

                #非金币奖励，全部固定值奖励
                if (!in_array($oneReward['rewardName'], self::MultiNumTypeReward)) {
                    $oneReward['isFixNum'] = '是';
                }

                #固定值奖励 1 按等级取值 2
                $rtype = ($oneReward['isFixNum'] == '是') ? 1 : 2;

                #奖励数值
                $num = $oneReward['rewardNum'];

                $rewardsTemp['prize_json'][] = [
                    "propsId" => intval($propsId),
                    "num" => intval($num),
                    "rtype" => $rtype,
                    "skin" => intval($skin)
                ];
            }

            $rewardsConf[] = $rewardsTemp;
        }

        return $rewardsConf;
    }

    /**
     *  解析额外字段配置
     * @param $extraConfArray
     * @return array
     */
    private function formatExcelExtraConf($extraConfArray): array
    {
        if (empty($extraConfArray)) {
            return [];
        }

        $fieldArr = [];
        foreach ($extraConfArray[0] as $field) {
            $fieldArr[] = $this->extraFieldsMap[$field];
        }

        $extraConf = [];
        for ($i = 1; $i < count($extraConfArray); $i++) {
            $oneExtra = [];
            foreach ($extraConfArray[$i] as $index => $item) {
                $oneExtra[$fieldArr[$index]] = $item;
            }
            $extraConf[] = $oneExtra;
        }

        return $extraConf;
    }

    //合并匹配数组，根据唯一ID
    private function matchGoodsConf(array $basicConf, array $rewardsConf, array $extraConf): array
    {
        //表中的所有字段
        $tableNameArr = explode('.', otable::giftPackGoods());
        $fieldsInfo = oo::commonOprDb('games')->getAll("select COLUMN_NAME from information_schema.COLUMNS
                    where table_name = '{$tableNameArr[1]}' and table_schema = '{$tableNameArr[0]}'");
        $fieldArr = array_column($fieldsInfo, 'COLUMN_NAME');

        $allRecordInfo = [];
        foreach ($basicConf as $oneBaseConf) {
            $matchTimes = 0;
            foreach ($rewardsConf as $index => $oneRewardConf) {
                //匹配上了
                if ($oneBaseConf['uuid'] == $oneRewardConf['uuid']) {
                    $matchTimes++;

                    $oneRecordInfo = $this->formatRecordInfo($fieldArr, array_merge($oneBaseConf, $oneRewardConf));

                    //额外字段
                    if (!empty($extraConf)) {
                        foreach ($extraConf as $oneExtra) {
                            if ($oneBaseConf['uuid'] == $oneExtra['uuid']) {
                                unset($oneExtra['uuid']);
                                $oneRecordInfo['other_json'] = $oneExtra;
                            }
                        }
                    }

                    if (empty($oneRecordInfo['max_level'])) {
                        $oneRecordInfo['max_level'] = $oneRecordInfo['min_level'];
                    }
                    unset($oneRecordInfo['id']);
                    $allRecordInfo[] = $oneRecordInfo;
                }
            }
            //没匹配到奖励
            if (empty($matchTimes)) {
                return [-1, "唯一ID：{$oneBaseConf['uuid']},未匹配到奖励"];
            }
        }

        return $allRecordInfo;
    }

    # 通过奖励名 解析奖励类型以及皮肤id
    private static function parseRewardTypeAndSkinId($propsConfig, $rewardName): array
    {
        foreach ($propsConfig as $oneProps) {
            //对比奖励名
            if ($oneProps['name'] == $rewardName) {
                return [$oneProps['id'], ''];
            }

            if (empty($oneProps['skinList'])) {
                continue;
            }

            //有皮肤列表时，对比皮肤名
            foreach ($oneProps['skinList'] as $oneSkin) {
                if ($oneSkin['name'] == $rewardName) {
                    return [$oneProps['id'], $oneSkin['id']];
                }
            }
        }

        return ['', ''];
    }

    # 通过奖励ID以及皮肤id 获取奖励名
    private function getRewardNameByTypeAndSkinId($propsId, $skin): string
    {
        foreach ($this->propsConfig as $oneProps) {
            //对比奖励ID
            if ($oneProps['id'] != $propsId) {
                continue;
            }

            if (empty($oneProps['skinList']) || empty($skin)) {
                return $oneProps['name'];
            }

            foreach ($oneProps['skinList'] as $oneSkin) {
                if ($oneSkin['id'] == $skin) {
                    return $oneSkin['name'];
                }
            }

            return '';
        }

        return '';
    }

    private function formatRecordInfo(array $fieldArr, array $recordInfo): array
    {
        $dataInfo = array_merge($this->payFixFields, $recordInfo);
        $recordFormat = [];
        foreach ($fieldArr as $field) {
            if (isset($dataInfo[$field])) {
                $recordFormat[$field] = $dataInfo[$field];
            }
        }
        return $recordFormat;
    }

    private function matchPrizeInfo($onePrize): string
    {
        $name = '';
        foreach ($this->propsConfig as $oneProp) {
            if ($oneProp['id'] == $onePrize['propsId']) {
                if (empty($onePrize['skin'])) {
                    $name = $oneProp['name'];
                    break;
                }
                foreach ($oneProp['skinList'] as $oneSkin) {
                    if ($onePrize['skin'] == $oneSkin['id']) {
                        $name = $oneSkin['name'];
                        break;
                    }
                }
            }
        }
        return $name . '：' . $onePrize['num'] . ($onePrize['rtype'] == 2 ? '倍' : '');
    }

    private function createImportConfig(string $imgName)
    {
        $payFixFields = [];

        switch ($imgName) {
            case self::IMG_NAME_SPRINT_PAY:
                $payFixFields = [
                    'desc' => '冲刺礼包商品',
                    'other_json' => '[]',
                    'ss_group' => '[]',
                    'status' => 1,
                    'gid' => 500000,
                ];

                break;
            case self::IMG_NAME_UNFITNESS_PAY:
                $payFixFields = [
                    'gid' => 510000,
                    'desc' => '体力不足礼包商品',
                    'ss_group' => '[]',
                    'other_json' => '[]',
                    'status' => 1,
                ];
                break;
            case self::IMG_NAME_UPGRADE_PAY:
                $payFixFields = [
                    'desc' => '升级礼包商品',
                    'other_json' => '[]',
                    'ss_group' => '[]',
                    'status' => 1,
                    'gid' => 520000,
                ];

                break;
        }

        $this->payFixFields = $payFixFields;
    }

    private function updateUpgradeJson()
    {
        $filename = 'GiftUpLevelConfig' . time() . '.json';

        $config['list'] = self::createUpgradeJson();

        //持续时间
        $duration = oo::commonOprModel('config')->getCon(self::IMG_NAME_UPGRADE_PAY, "duration") ?? 0;

        $config['expiryTime'] = 60 * $duration;
        oo::commonOprModel("json")->SaveJson($config, $filename);
        oo::commonOprModel("json")->setStaticConfig("GiftUpLevelConfig", $filename);
    }

    private function createUpgradeJson(): array
    {
        $imgName = self::IMG_NAME_UPGRADE_PAY;
        $gType = $this->gTypeMap[$imgName];

        $sql = "SELECT * FROM {$this->giftPackGoodsTable} where g_type={$gType};";
        $tableData = oo::commonOprDb('games')->getAll($sql, 1);

        $jsonInfo = [];
        foreach ($tableData as $index => $oneRecord) {
            $jsonInfo[] = [
                "section_id" => $index,
                "gid" => intval($oneRecord['gid']),
                "start_level" => intval($oneRecord['min_level']),
                "end_level" => intval($oneRecord['max_level']),
                "multi" => intval(json_decode($oneRecord['other_json'], true)['showMultiple'])
            ];
        }

        return $jsonInfo;
    }

    // 商城免费礼物列表
    public static function DailyFreeConf(): array
    {
        $tb = Otable::dailyFreeConf();
        $sql = "SELECT `id`,`name`,`resource`,`start_at`,`end_at` from {$tb}";
        return oo::commonOprDb('slavedb')->getAll($sql);
    }

    // 商城免费礼物详情
    public static function DailyFreeRewardConf($id): array
    {
        $tb = Otable::dailyFreeRewardConf();
        $sql = "SELECT `day`,`prize` from {$tb} where `id`={$id}";
        $rewardConfArr = oo::commonOprDb('slavedb')->getAll($sql);

        $rewardConf = [];
        for ($i = 0; $i < count($rewardConfArr); $i++) {
            $oneReward = $rewardConfArr[$i];
            $prize = json_decode($oneReward['prize'], true);

            for ($j = 0; $j < count($prize); $j++) {
                $rewardConf[$j + 1]['stages'][$oneReward['day']] = [
                    'day' => $oneReward['day'],
                    'prize' => $prize[$j]['reward'],
                ];
                $rewardConf[$j + 1]['lvStart'] = $prize[$j]['lvStart'];
                $rewardConf[$j + 1]['lvEnd'] = $prize[$j]['lvEnd'];
            }
        }

        return $rewardConf;
    }

    // 商城免费礼物更新
    public static function DailyFreeConfUpdate(array $postData): array
    {
        // 更新基础配置
        $tb = Otable::dailyFreeConf();
        $sql = funs::db_updateSQL($tb,
            [
                'name' => $postData['name'],
                'resource' => $postData['resource'],
                'start_at' => $postData['start_at'],
                'end_at' => $postData['end_at'],
            ], "id={$postData['id']}");
        $res = oo::commonOprDb('games')->query($sql);
        if (!$res) {
            return [-1, '基础配置更新失败'];
        }

        $rewardConf = [];

        // 奖励配置
        foreach ($postData['stages'] as $lvIdx => $oneItem) {
            foreach ($oneItem['stages'] as $rIdx => $oneReward) {
                if (empty($oneReward['prize'])) {
                    return [-1, "奖励配置为空"];
                }

                $rewardConf[$oneReward['day']][] = [
                    'lvStart' => $oneItem['lvStart'],
                    'lvEnd' => $oneItem['lvEnd'],
                    'reward' => $oneReward['prize'],
                ];
            }
        }

        $tb = Otable::dailyFreeRewardConf();
        $sql = "Select DISTINCT `day` from {$tb}";
        $ret = oo::commonOprDb('slavedb')->getAll($sql);
        $days = array_column($ret, 'day');
        foreach ($rewardConf as $day => $reward) {
            foreach ($days as $i => $dayV) {
                if ($day == $dayV) {
                    $sql = funs::db_updateSQL($tb,
                        [
                            'prize' => json_encode($reward, JSON_NUMERIC_CHECK)
                        ], "id = {$postData['id']} and day = {$day}");
                    unset($days[$i]);
                    break;
                }

                if ($i + 1 == count($days)) {
                    $sql = funs::db_insertSQL($tb,
                        [
                            'id' => $postData['id'],
                            'day' => $day,
                            'prize' => json_encode($reward, JSON_NUMERIC_CHECK)
                        ]);
                }
            }
            $res = oo::commonOprDb('games')->query($sql);
            if (!$res) {
                return [-1, '奖励配置数据库保存失败'];
            }
        }

        // 删除多余的奖励记录
        if (!empty($days)) {
            foreach ($days as $day) {
                $sql = "delete from {$tb} Where `id`={$postData['id']} and `day` = {$day}";
                $res = oo::commonOprDb('games')->query($sql);
                if (!$res) {
                    return [-1, '奖励配置数据库删除失败'];
                }
            }
        }

        return [0, '操作成功'];
    }

    // 商城免费礼物导入
    public static function DailyFreeRewardImport(array $dataArray): array
    {

        $sheet1Arr = $dataArray[0] ?? [];
        $sheet2Arr = $dataArray[1] ?? [];
        $sheet3Arr = $dataArray[2] ?? [];

        if (empty($sheet1Arr) || empty($sheet2Arr) || empty($sheet3Arr)) {
            return [-1, '导入的工作表数量不足'];
        }

        $basicConf = [];

        // 基础配置解析
        foreach ($sheet1Arr as $oneRow) {
            switch ($oneRow[0]) {
                case "名称":
                    $basicConf['name'] = $oneRow[1] ?? '';
                    break;
                case "资源文件夹名":
                    $basicConf['resource'] = $oneRow[1] ?? '';
                    break;
                case "开始日期":
                    $basicConf['start_at'] = $oneRow[1] ?? '';
                    break;
                case "结束日期":
                    $basicConf['end_at'] = $oneRow[1] ?? '';
                    break;
            }
        }

        if (empty($basicConf['start_at']) || empty($basicConf['end_at'])) {
            return [-1, '活动时间范围未确定'];
        }

        $layer = [];

        // 奖励配置解析
        for ($i = 1; $i < count($sheet2Arr); $i++) {
            $oneRow = $sheet2Arr[$i];
            if (empty(array_filter($oneRow))) continue;

            list($idx, $lvMin, $lvMax) = $oneRow;

            $layer[] = ['id' => $idx, 'lvStart' => $lvMin, 'lvEnd' => $lvMax];
        }

        $rewardArrMap = [];
        $propsConfig = oo::commonOprModel('config')->getPropsConfigFromLocal();

        for ($i = 1; $i < count($sheet3Arr); $i++) {
            $oneRow = $sheet3Arr[$i];
            if (empty(array_filter($oneRow))) continue;

            list($idx, $day,
                $reward1, $isFix1, $num1,
                $reward2, $isFix2, $num2,
                $reward3, $isFix3, $num3,) = $oneRow;

            $rewardsTemp = [];
            for ($j = 1; $j <= 3; $j++) {
                if (empty(${'reward' . $j})) continue;

                list($propsId, $skin) = self::parseRewardTypeAndSkinId($propsConfig, ${"reward{$j}"});
                if (empty($propsId)) {
                    return [-1, "唯一ID:{$idx} 未找到奖励项：" . ${"reward{$j}"}];
                }

                #非金币奖励，全部固定值奖励
                if (!in_array(${"isFix{$j}"}, self::MultiNumTypeReward)) {
                    ${"isFix{$j}"} = '是';
                }

                #固定值奖励 1 按等级取值 2
                $rtype = (${"isFix{$j}"} == '是') ? 1 : 2;

                #奖励数值
                $num = ${"num{$j}"};

                $rewardsTemp[] = [
                    "propsId" => intval($propsId),
                    "num" => intval($num),
                    "rtype" => $rtype,
                    "skin" => intval($skin)
                ];
            }

            $rewardArrMap[$idx][$day] = $rewardsTemp;
        }

        $rewardConf = [];
        // 奖励和分组匹配
        foreach ($rewardArrMap as $id => $prizeMap) {

            foreach ($layer as $one) {
                if (empty($rewardArrMap[$one['id']])) {
                    return [-1, "唯一ID: {$one['id']}分组未配置奖励项"];
                }
                if ($id != $one['id']) {
                    continue;
                }

                foreach ($prizeMap as $day => $reward) {
                    if (empty($reward)) {
                        return [-1, "奖励配置为空"];
                    }
                    $rewardConf[$day][] = [
                        'lvStart' => $one['lvStart'],
                        'lvEnd' => $one['lvEnd'],
                        'reward' => $reward,
                    ];
                }
            }
        }

        // 新增基础配置
        $tb = Otable::dailyFreeConf();
        $sql = funs::db_insertSQL($tb, $basicConf);
        $rus = oo::commonOprDb('games')->query($sql);
        if (!$rus) {
            return [-1, '基础配置数据库保存失败'];
        }
        $insertId = $rus ? oo::commonOprDb('games')->insertID() : 0;

        // 奖励配置
        $tb = Otable::dailyFreeRewardConf();
        foreach ($rewardConf as $day => $reward) {
            $sql = funs::db_insertSQL($tb, ['id' => $insertId, 'day' => $day, 'prize' => json_encode($reward, JSON_NUMERIC_CHECK)]);
            $res = oo::commonOprDb('games')->query($sql);
            if (!$res) {
                return [-1, '奖励配置数据库保存失败'];
            }
        }

        return [0, '操作成功'];
    }

    // 通过活动类型获取活动商品gtype
    public function getGtypeByActType(string $actType):int{
        $tb = otable::gTypeGoods();
        $ret = oo::commonOprDb('slavedb')->getOne("SELECT * FROM {$tb} Where act_type='".$actType."'");
        if($ret){
            return $ret['g_type'];
        }
        return 0;
    }
}